import uuid
from abc import ABC, abstractmethod

import mne
from numpy import ndarray
from typing import Any, Tuple, List
from sklearn.base import BaseEstimator
from nptyping import NDArray


class ID:
	"""
	DESCRIPTION
	-----------
	ID

	"""

	def __init__(self, objtype, number=None, name=None, description=None):
		"""
		DESCRIPTION
		-----------
		The Constructor for ID

		"""
		self._type = objtype
		self.classifier = None
		if number is None:
			number = uuid.uuid1().int
		self._number = number
		if name is None:
			name = objtype.__name__
		self._name = name
		if description is None:
			description = ''
		self._description = description

	def __repr__(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return (f'{self.__class__.__name__}('
		        f'{self._number!r}, {self._name!r})')

	def __str__(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return str(self.get_number())

	def __eq__(self, other):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		if isinstance(other, ID):
			return self.get_number() == other.get_number()
		elif isinstance(other, int):
			return self.get_number() == other
		else:
			return super().__eq__(other)

	def __hash__(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self.get_number()

	def get_number(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._number

	def get_name(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._name

	def get_description(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._description

	def set_name(self, name):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		self._name = name

	def set_description(self, description):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		self._description = description

	def get_type(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._type


class BaseObject(ABC):

	def __init__(self, id: ID):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		self._id = id

	def __hash__(self):
		return self.get_id().__hash__()

	def get_id(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._id

	def set_id(self, id: ID):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		self._id = id


class Database:

	def __init__(self, database: dict = None):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		self._database = database if database is not None else dict()

	def add(self, *objs: BaseObject):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		for obj in objs:
			self._database[obj.get_id()] = obj

	def get(self, *ids: ID or int):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		output = []
		for id in ids:
			output.append(self._database.get(id))
		return output

	def pop(self, *ids: ID or int):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		output = []
		for id in ids:
			output.append(self._database.pop(id))
		return output

	def get_database_dict(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._database


class Stage(BaseObject, ABC):
	"""
	DESCRIPTION
	-----------
	ID

	"""

	def __init__(self, id: ID, database: Database, inputs: Tuple[ID, ...], outputs: Tuple[ID, ...]):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		super().__init__(id)
		self._database = database
		self._inputs = inputs
		self._outputs = outputs
		self._params = dict()
		self.finished = False

	def set_params(self, *args, **kwargs):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		pass

	def get_params(self, *args):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		if len(args) == 0:
			return self._params.copy()
		else:
			return {key: self._params[key] for key in args}

	@abstractmethod
	def do_task(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		pass

	def _get_input(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		from copy import deepcopy
		return deepcopy(self._database.get(*self._inputs))

	def _set_output(self, output, output_id):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		output.set_id(output_id)
		# output.add_process(self)
		self._database.add(output)

	def _finish(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		self.finished = True


class BaseData(BaseObject, ABC):
	"""
	DESCRIPTION
	-----------
	ID

	"""

	def __init__(self, id: ID, data: Any, processes=None):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		super().__init__(id)
		self._data = data

	# self._processes = list() if processes is None else processes

	# def set_processes(self, processes: List[Stage]):
	# 	self._processes = processes

	# def add_process(self, process: Stage):
	# self._processes.append(process)

	# def get_processes(self):
	# 	return self._processes

	def get_data(self):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return self._data

	def get_feature(self, dim: int):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		function_name = "_get_" + str(dim) + "d_feature"
		if hasattr(self, function_name):
			return getattr(self, function_name)()

	def save_obj(self, output: str):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		import pickle
		file_to_store = open(output, "wb")
		pickle.dump(self, file_to_store)
		file_to_store.close()


class EEGData(BaseData, ABC):
	"""
	DESCRIPTION
	-----------
	ID

	"""

	def __init__(self, id: ID, data: Any, processes=None):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		super().__init__(id, data, processes)

	def get_labels(self, stim_channel_name: str = "Stim"):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		stim = self._data.get_data(picks=stim_channel_name)
		return stim[stim != 0]


class FeatureData(BaseData, ABC):
	"""
	DESCRIPTION
	-----------
	ID

	"""

	def __init__(self, id: ID, data: NDArray, label: NDArray[(Any,)], processes=None):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		super().__init__(id, data, processes)
		self._label = label


class RawData(EEGData):
	"""
	DESCRIPTION
	-----------
	ID

	"""

	def __init__(self, id: ID, data: mne.io.RawArray, processes=None):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		super().__init__(id, data, processes)

	def _get_1d_feature(self) -> Tuple[NDArray[(Any, Any)], NDArray[(Any,)]]:
		pass

	def _get_2d_feature(self) -> Tuple[NDArray[(Any, Any, Any)], NDArray[(Any,)]]:
		pass


class EpochsData(EEGData):

	def __init__(self, id: ID, data: mne.Epochs, processes=None):
		super().__init__(id, data, processes)

	def _get_1d_feature(self) -> Tuple[NDArray[(Any, Any)], NDArray[(Any,)]]:
		pass

	def _get_2d_feature(self) -> Tuple[NDArray[(Any, Any, Any)], NDArray[(Any,)]]:
		pass


class OneDFeature(FeatureData):

	def __init__(self, id: ID, data: NDArray[(Any, Any)], label: NDArray[(Any,)], processes=None):
		super().__init__(id, data, label, processes)

	def get_feature(self, dim: int = 1):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return super().get_feature(dim)

	def _get_1d_feature(self) -> Tuple[NDArray[(Any, Any)], NDArray[(Any,)]]:
		return self.get_data(), self._label


class TwoDFeature(FeatureData):

	def __init__(self, id: ID, data: NDArray[(Any, Any, Any)], label: NDArray[(Any,)], processes=None):
		super().__init__(id, data, label, processes)

	def get_feature(self, dim: int = 2):
		"""
		DESCRIPTION
		-----------
		ID

		"""
		return super().get_feature(dim)

	def get_2d_feature(self) -> Tuple[NDArray[(Any, Any, Any)], NDArray[(Any,)]]:
		return self.get_data(), self._label


class BaseModel(BaseObject):

	def __init__(self, id: ID):
		super().__init__(id)


class SKModel(BaseModel):

	def __init__(self, id: ID, model: BaseEstimator, classifier):
		super().__init__(id)
		self._model = model
		self._classifier = classifier


class BaseResult(BaseObject):

	def __init__(self, id: ID):
		super().__init__(id)


class SimpleResult(BaseResult):

	def __init__(self, id: ID, result, classifier):
		super().__init__(id)
		self.result = result



import os
import pickle
import numpy as np
from mne.io import RawArray
from mne import create_info
from scipy.io import loadmat
from mne.channels import make_standard_montage
from bci_lib import RawData
from bci_lib.DataTypes import BaseData, ID


class Dataset:
    _BASE_LOCAL_LOCATION = '/content/'
    _BASE_LOCAL_LOCATION_reset = _BASE_LOCAL_LOCATION

    @classmethod
    def change_BASE_LOCAL_LOCATION(cls, new_location):
        cls._BASE_LOCAL_LOCATION = new_location

    @classmethod
    def reset_BASE_LOCAL_LOCATION(cls):
        cls._BASE_LOCAL_LOCATION = cls._BASE_LOCAL_LOCATION_reset

    @staticmethod
    def get_file_location(file_name):
        return Dataset._BASE_LOCAL_LOCATION + file_name

    @staticmethod
    def get_current_file_location():
        return Dataset.get_file_location("current_file")

    @staticmethod
    def download(url: str, to: str):
        if url.lower().startswith(("http:", "https:")):
            import requests
            try:
                r = requests.get(url)
            except requests.ConnectionError:
                print("!!! Failed to download data !!!")
            else:
                if r.status_code != requests.codes.ok:
                    print("!!! Failed to download data !!!")
                else:
                    with open(to, "wb") as fid:
                        fid.write(r.content)
        elif url.lower().startswith("ftp:"):
            import shutil
            import urllib.request as request
            from contextlib import closing

            with closing(request.urlopen(url)) as r:
                with open(to, 'wb') as fid:
                    shutil.copyfileobj(r, fid)
        else:
            from shutil import copyfile
            copyfile(url, to)

    @staticmethod
    def delete_file(file):
        if os.path.exists(file):
            os.remove(file)
        else:
            print("The file does not exist")

    @staticmethod
    def load(data_info: dict, save_in_cache: bool = True) -> BaseData:
        # check is in local_location if true load this else call download_and_prepare function
        download_url = data_info['FUNCTION'].__func__(data_info['FILE'])
        local_file_path = Dataset.get_file_location(data_info['LOCAL'])
        prepare_func = data_info['PREPARE_FUNC'].__func__
        current_file_path = Dataset.get_current_file_location()

        if os.path.isfile(local_file_path):
            file_to_read = open(local_file_path, "rb")
            loaded_object = pickle.load(file_to_read)
            file_to_read.close()
        else:
            Dataset.download(download_url, current_file_path)
            loaded_object = prepare_func(current_file_path)
            Dataset.delete_file(current_file_path)
            if save_in_cache:
                loaded_object.save_obj(local_file_path)
        return loaded_object

    class Cho2017:
        _FOLDER_URL = "ftp://parrot.genomics.cn/gigadb/pub/10.5524/100001_101000/100295/mat_data/"
        _FOLDER_URL_reset = _FOLDER_URL

        @classmethod
        def change_FOLDER_URL(cls, new_url):
            cls._FOLDER_URL = new_url

        @classmethod
        def reset_FOLDER_URL(cls):
            cls._FOLDER_URL = cls._FOLDER_URL_reset

        @staticmethod
        def get_FILE_URL(file_name):
            return Dataset.Cho2017._FOLDER_URL + file_name

        @staticmethod
        def prepare(current_file: str, ):
            data = loadmat(current_file, squeeze_me=True, struct_as_record=False,
                           verify_compressed_data_integrity=False)['eeg']
            eeg_ch_names = ['Fp1', 'AF7', 'AF3', 'F1', 'F3', 'F5', 'F7', 'FT7',
                            'FC5', 'FC3', 'FC1', 'C1', 'C3', 'C5', 'T7', 'TP7',
                            'CP5', 'CP3', 'CP1', 'P1', 'P3', 'P5', 'P7', 'P9',
                            'PO7', 'PO3', 'O1', 'Iz', 'Oz', 'POz', 'Pz', 'CPz',
                            'Fpz', 'Fp2', 'AF8', 'AF4', 'AFz', 'Fz', 'F2', 'F4',
                            'F6', 'F8', 'FT8', 'FC6', 'FC4', 'FC2', 'FCz', 'Cz',
                            'C2', 'C4', 'C6', 'T8', 'TP8', 'CP6', 'CP4', 'CP2',
                            'P2', 'P4', 'P6', 'P8', 'P10', 'PO8', 'PO4', 'O2']
            emg_ch_names = ['EMG1', 'EMG2', 'EMG3', 'EMG4']
            ch_names = eeg_ch_names + emg_ch_names + ['Stim']
            ch_types = ['eeg'] * 64 + ['emg'] * 4 + ['stim']
            montage = make_standard_montage('standard_1005')

            imagery_left = data.imagery_left - \
                data.imagery_left.mean(axis=1, keepdims=True)
            imagery_right = data.imagery_right - \
                data.imagery_right.mean(axis=1, keepdims=True)
            eeg_data_l = np.vstack([imagery_left * 1e-6, data.imagery_event])
            eeg_data_r = np.vstack(
                [imagery_right * 1e-6, data.imagery_event * 2])
            eeg_data = np.hstack([eeg_data_l, np.zeros(
                (eeg_data_l.shape[0], 500)), eeg_data_r])

            info = create_info(ch_names=ch_names,
                               ch_types=ch_types, sfreq=data.srate)
            raw = RawArray(data=eeg_data, info=info, verbose=False)
            raw.set_montage(montage)

            raw_data = RawData(id=ID(RawData), data=raw)
            return raw_data

        s01 = {'FILE': 's01.mat', 'LOCAL': 'Cho2017_s01',
               'FUNCTION': get_FILE_URL, 'PREPARE_FUNC': prepare}



from .Stages.Classification.ML import CreateModel, Train, Test
from .Stages.LoadData import LoadFromDataset, LoadRaw, LoadEpochs
from typing import Tuple
from .DataTypes import BaseObject, Database, ID, Stage, SKModel, BaseResult, RawData, EpochsData
from .Stages.Preprocess import TestTrainSplit
from .Stages.FeatureExtraction import PCA,ICA


class SingleRun(BaseObject):
	"""
	DESCRIPTION
	-----------
	Single run

	"""

	def __init__(self):
		"""
		DESCRIPTION
		-----------
		The Constructor for SingleRun

		"""
		my_id = ID(SingleRun)
		super().__init__(my_id)
		self._database = Database()
		self.pipeline = []
		self.train_ids = []
		self.base_data_ids = []
		self.test_ids = []
		self.model_ids = []
		self.train_pipeline = []
		self.test_pipeline = []

	def _get_stage(self, stage_idx=-1):
		"""
		DESCRIPTION
		-----------
		_get_stage

		Parameters
		-----------
		stage_idx : int | -1

		"""
		return self.pipeline[stage_idx]

	def _add_to_pipeline(self, my_stage):
		"""
		DESCRIPTION
		-----------
		_add_to_pipeline

		Parameters
		-----------
		my_stage : int

		"""
		self.pipeline.append(my_stage)

	def add_stage(self, stage, stage_idx=-1):
		"""
		DESCRIPTION
		-----------
		add_stage

		Parameters
		-----------
		stage_idx : int | -1

		"""
		stage_id = ID(stage)
		if len(self.train_ids) == 0:
			if stage_idx != -1:
				stage_output_id = ID(
					stage.in_out[self._get_stage(stage_idx)._outputs[0].get_type()])
				my_stage = stage(stage_id, self._database, self._get_stage(stage_idx)._outputs,
				                 (stage_output_id,))
			elif stage is CreateModel:
				stage_output_id = ID(SKModel)
				my_stage = stage(stage_id, self._database, (stage_output_id,))
				self.model_ids.append(stage_output_id)
			elif stage is LoadFromDataset:
				stage_output_id = ID(RawData)
				my_stage = stage(stage_id, self._database, (stage_output_id,))
				self.base_data_ids.append(stage_output_id)
			elif stage is LoadRaw:
				def load_info(mark_as_test: bool = False):
					if not isinstance(mark_as_test, bool):
						raise TypeError(
							'mark_as_test parameter should be boolean but it is {}'.format(type(mark_as_test)))
					if mark_as_test:
						stage_output_id_test = ID(EpochsData)
						my_stage_test = stage(
							stage_id, self._database, (stage_output_id_test,))
						self.test_ids.append(stage_output_id_test)
						self.train_ids.append(self.base_data_ids[-1])
						self._add_to_pipeline(my_stage_test)
						return my_stage_test
					else:
						stage_output_id = ID(EpochsData)
						my_stage = stage(
							stage_id, self._database, (stage_output_id,))
						self._add_to_pipeline(my_stage)
						self.base_data_ids.append(stage_output_id)
						return my_stage

				return load_info
			elif stage is LoadEpochs:
				def load_info(mark_as_test: bool = False):
					if not isinstance(mark_as_test, bool):
						raise TypeError('mark_as_test parameter should be boolean but it is {}'.format(
							type(mark_as_test)))
					if mark_as_test:
						stage_output_id_test = ID(EpochsData)
						my_stage_test = stage(
							stage_id, self._database, (stage_output_id_test,))
						self.test_ids.append(stage_output_id_test)
						self.train_ids.append(self.base_data_ids[-1])
						self._add_to_pipeline(my_stage_test)
						return my_stage_test
					else:
						stage_output_id = ID(EpochsData)
						my_stage = stage(
							stage_id, self._database, (stage_output_id,))
						self._add_to_pipeline(my_stage)
						self.base_data_ids.append(stage_output_id)
						return my_stage

				return load_info

			elif stage is Train:
				stage_output_id = ID(SKModel)
				my_stage = stage(stage_id, self._database, (self.base_data_ids[-1],),
				                 (stage_output_id,), self.model_ids[-1])
				self.model_ids.append(stage_output_id)
			elif stage is Test:
				stage_output_id = ID(BaseResult)
				my_stage = stage(stage_id, self._database, (self.base_data_ids[-1],),
				                 (stage_output_id,), self.model_ids[-1])
				self.base_data_ids.append(stage_output_id)
			elif stage is TestTrainSplit:
				stage_output_id_train = ID(EpochsData)
				stage_output_id_test = ID(EpochsData)
				my_stage = stage(stage_id, self._database, (self.base_data_ids[-1],),
				                 (stage_output_id_train, stage_output_id_test))
				self.train_ids.append(stage_output_id_train)
				self.test_ids.append(stage_output_id_test)
			else:
				stage_output_id = ID(
					stage.in_out[self.base_data_ids[-1].get_type()])
				my_stage = stage(stage_id, self._database, (self.base_data_ids[-1],),
				                 (stage_output_id,))
				self.base_data_ids.append(stage_output_id)
			# stage in_out looks like this -> {RawData: RawData, EpochsData:EpochsData}
			self._add_to_pipeline(my_stage)
			return my_stage
		else:
			if stage is CreateModel:
				stage_output_id = ID(SKModel)
				my_stage = stage(stage_id, self._database, (stage_output_id,))
				self.model_ids.append(stage_output_id)
				self._add_to_pipeline(my_stage)
				return my_stage
			elif stage is LoadFromDataset:
				stage_output_id = ID(RawData)
				my_stage = stage(stage_id, self._database, (stage_output_id,))
				self._add_to_pipeline(my_stage)
				return my_stage
			elif stage is LoadRaw:
				raise Exception(
					'You have test and train datas, you cant use load again')
			elif stage is LoadEpochs:
				raise Exception(
					'You have test and train datas, you cant use load again')
			elif stage is TestTrainSplit:
				raise Exception('error using TestTrainSplit twice!')
			elif stage is Train:
				stage_output_id = ID(SKModel)
				my_stage = stage(stage_id, self._database, (self.train_ids[-1],),
				                 (stage_output_id,), self.model_ids[-1])
				self.train_pipeline.append(my_stage)
				self.model_ids.append(stage_output_id)
				return my_stage
			elif stage is Test:
				stage_output_id = ID(BaseResult)
				my_stage = stage(stage_id, self._database, (self.test_ids[-1],),
				                 (stage_output_id,), self.model_ids[-1])
				self.test_pipeline.append(my_stage)
				return my_stage
			elif (stage is PCA) or (stage is ICA):
				stage_output_id_train = ID(
					stage.in_out[self.train_ids[-1].get_type()])
				my_stage_train = stage(stage_id, self._database, (self.train_ids[-1],),
				                       (stage_output_id_train,))
				self.train_ids.append(stage_output_id_train)
				self.train_pipeline.append(my_stage_train)

				stage_output_id_test = ID(
					stage.in_out[self.test_ids[-1].get_type()])
				my_stage_test = stage(stage_id, self._database, (self.test_ids[-1],),
				                      (stage_output_id_test,), mark_as_test=True)
				my_stage_test.cache = my_stage_train.cache
				self.test_ids.append(stage_output_id_test)
				self.test_pipeline.append(my_stage_test)

				my_stage_test._params = my_stage_train._params
				return my_stage_train
			else:
				stage_output_id_train = ID(
					stage.in_out[self.train_ids[-1].get_type()])
				my_stage_train = stage(stage_id, self._database, (self.train_ids[-1],),
				                       (stage_output_id_train,))
				self.train_ids.append(stage_output_id_train)
				self.train_pipeline.append(my_stage_train)

				stage_output_id_test = ID(
					stage.in_out[self.test_ids[-1].get_type()])
				my_stage_test = stage(stage_id, self._database, (self.test_ids[-1],),
				                      (stage_output_id_test,))
				self.test_ids.append(stage_output_id_test)
				self.test_pipeline.append(my_stage_test)

				my_stage_test._params = my_stage_train._params
				return my_stage_train

	def set_stage_params(self, stage_idx=-1):
		"""
		DESCRIPTION
		-----------
		set_stage_params

		Parameters
		-----------
		stage_idx : int | -1

		"""
		return self.pipeline[stage_idx].set_params

	def do_task(self):
		"""
		DESCRIPTION
		-----------
		do_task

		"""
		for stage in self.pipeline:
			stage.do_task()
		for stage in self.train_pipeline:
			stage.do_task()
		for stage in self.test_pipeline:
			stage.do_task()


class Pipeline(BaseObject):
	"""
	DESCRIPTION
	-----------
	The Constructor for SingleRun

	Attributes
	-----------
	id : ID

	edges : Tuple[Tuple[Stage, Stage or None], ...] | None

	"""

	def __init__(self, id: ID, edges: Tuple[Tuple[Stage, Stage or None], ...] = None):
		"""
		DESCRIPTION
		-----------
		The Constructor for Pipeline

		Parameters
		-----------
		id : ID

		edges : Tuple[Tuple[Stage, Stage or None], ...] | None

		"""
		super().__init__(id)
		self._edges = dict()
		self._edges[None] = list()  # for root
		self._incoming_edge_count = dict()
		if edges is not None:
			for edge in edges:
				self.add_edge(dst=edge[0], src=edge[1])

	def add_edge(self, dst: Stage, src: Stage = None):
		"""
		DESCRIPTION
		-----------
		add_edge

		Parameters
		-----------
		dst : bci_lib.Stage

		src : bci_lib.Stage | None

		"""
		if src not in self._edges:
			self._edges[src] = list()
			if src is not None:
				self.add_edge(dst=src, src=None)
		self._edges[src].append(dst)
		if dst not in self._incoming_edge_count:
			self._incoming_edge_count[dst] = 0
		self._incoming_edge_count[dst] += 1

	def get_edges(self) -> Tuple[Tuple[Stage, Stage or None], ...]:
		return tuple((dst, src) for src in self._edges for dst in self._edges[src])

	def get_topo_sort(self):
		topo_order = list()
		roots = self._edges[None].copy()
		while roots:
			src = roots.pop(0)
			# Iterate through the adjacency list
			if src in self._edges:
				for stage in self._edges[src]:
					self._incoming_edge_count[stage] -= 1
					if self._incoming_edge_count[stage] == 0:
						roots.append(stage)
			topo_order.append(src)
		del roots
		return topo_order



from distutils.core import setup

setup(
	name='bci_lib',
	packages=['bci_lib'],
	version='0.0.7',
	license='MIT',
	description='A useful and easy to use tool for building bci pipelines.',
	author='Sahand Sadeghpour',
	author_email='sahand.2k.ss@gmail.com',
	url='https://github.com/SahandSadeghpour/bci_lib',
	download_url='https://github.com/SahandSadeghpour/bci_lib/archive/v0.0.7.tar.gz',
	keywords=['bci', 'pipelines', 'preprocessing', 'classification'],
	install_requires=[
		'mne~=0.20.4',
		'numpy~=1.18.4',
		'sklearn~=0.0',
		'scikit-learn~=0.23.2',
		'scipy~=1.4.1',
		'nptyping~=1.3.0',
	],
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Intended Audience :: Developers',
		'Intended Audience :: Science/Research',
	    'Topic :: Software Development :: Build Tools',
	    'License :: OSI Approved :: MIT License',
	    'Programming Language :: Python :: 3',
	    'Programming Language :: Python :: 3.4',
	    'Programming Language :: Python :: 3.5',
	    'Programming Language :: Python :: 3.6',
	],
)
